import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

public class DN09 {
    public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("Ukaz manjka");
            return;
        }

        Tekmovanje tekmovanje = Tekmovanje.izDatotek(args[1], args[2]);
        assert tekmovanje != null;

        switch (args[0]) {
            case "izpisiTekmovanje":
                tekmovanje.izpisiTekmovalce();
                System.out.println();
                tekmovanje.izpisiGlasove();
                break;

            case "izpisiTocke":
                tekmovanje.izpisiGlasove();
                break;

            case "najboljse":
                tekmovanje.izpisiRezultateUrejeno(Integer.parseInt(args[3]));
                break;

            case "utezeno":
                tekmovanje.setKriterij(new UtezeniKriterij(Float.parseFloat(args[4]), Float.parseFloat(args[5])));
                tekmovanje.izpisiRezultateUrejeno(Integer.parseInt(args[3]));
                break;

            default:
                System.out.println("Neznan ukaz " + args[0]);
        }
    }
}

class Tekmovalec {
    private String drzava, izvajalec, naslovPesmi;

    Tekmovalec(String drzava, String izvajalec, String naslovPesmi) {
        this.drzava = drzava;
        this.izvajalec = izvajalec;
        this.naslovPesmi = naslovPesmi;
    }

    String getDrzava() {
        return drzava;
    }

    String getIzvajalec() {
        return izvajalec;
    }

    String getNaslovPesmi() {
        return naslovPesmi;
    }

    @Override
    public String toString() {
        return String.format("(%s) %s - %s", drzava, izvajalec, naslovPesmi);
    }
}

class Glas {
    private String odDrzave, zaDrzavo;
    private int stTock;

    Glas(String odDrzave, String zaDrzavo, int stTock) {
        this.odDrzave = odDrzave;
        this.zaDrzavo = zaDrzavo;
        this.stTock = stTock;
    }

    public String getOdDrzave() {
        return odDrzave;
    }

    public String getZaDrzavo() {
        return zaDrzavo;
    }

    public int getStTock() {
        return stTock;
    }

    @Override
    public String toString() {
        return String.format("%s --%dt-> %s", odDrzave, stTock, zaDrzavo);
    }
}

class Tekmovanje {
    private ArrayList<Tekmovalec> seznamTekmovalcev;
    private ArrayList<Glas> seznamGlasov;
    private Kriterij kriterij;
    private boolean urejeno = false;

    Tekmovanje(ArrayList<Tekmovalec> seznamTekmovalcev, ArrayList<Glas> seznamGlasov) {
        this.seznamTekmovalcev = seznamTekmovalcev;
        this.seznamGlasov = seznamGlasov;
        this.kriterij = new OsnovniKriterij();
    }

    static Tekmovanje izDatotek(String datotekaTekmovalci, String datotekaGlasovi) {
        Scanner scannerTekmovalci, scannerGlasovi;

        try {
            scannerTekmovalci = new Scanner(new File(datotekaTekmovalci));
            scannerGlasovi = new Scanner(new File(datotekaGlasovi));
        } catch (FileNotFoundException e) {
            return null;
        }

        scannerTekmovalci.nextLine();
        scannerGlasovi.nextLine();

        ArrayList<Tekmovalec> seznamTekmovalcev = new ArrayList<>();
        while (scannerTekmovalci.hasNextLine()) {
            String[] fields = scannerTekmovalci.nextLine().split(";");
            seznamTekmovalcev.add(new Tekmovalec(fields[1], fields[2], fields[3]));
        }

        ArrayList<Glas> seznamGlasov = new ArrayList<>();
        while (scannerGlasovi.hasNextLine()) {
            String[] fields = scannerGlasovi.nextLine().split(";");
            switch (fields.length) {
                case 5:
                    seznamGlasov.add(new Glas(fields[2], fields[3], Integer.parseInt(fields[4])));
                    break;
                case 6:
                    seznamGlasov.add(new LocenGlas(fields[2], fields[3], Integer.parseInt(fields[4]),
                            Integer.parseInt(fields[5]), 0));
                    break;
                case 7:
                    seznamGlasov.add(new LocenGlas(fields[2], fields[3], Integer.parseInt(fields[4]),
                            Integer.parseInt(fields[5]), Integer.parseInt(fields[6])));
                    break;
            }
        }

        return new Tekmovanje(seznamTekmovalcev, seznamGlasov);
    }

    public ArrayList<Tekmovalec> getSeznamTekmovalcev() {
        return seznamTekmovalcev;
    }

    public ArrayList<Glas> getSeznamGlasov() {
        return seznamGlasov;
    }

    public void setKriterij(Kriterij kriterij) {
        this.kriterij = kriterij;
        urejeno = false;
    }

    void izpisiTekmovalce() {
        if (seznamTekmovalcev.isEmpty()) {
            System.out.println("Seznam tekmovalcev je prazen.");
            return;
        }
        System.out.println("Seznam tekmovalcev:");
        for (Tekmovalec tekmovalec : seznamTekmovalcev) {
            System.out.println(tekmovalec);
        }
    }

    void izpisiGlasove() {
        if (seznamGlasov.isEmpty()) {
            System.out.println("Seznam glasov je prazen.");
            return;
        }
        System.out.println("Seznam glasov:");
        for (Glas glas : seznamGlasov) {
            System.out.println(glas);
        }
    }

    int steviloTock(String drzava) {
        return kriterij.steviloTock(this, drzava);
    }

    void izpisiTocke() {
        if (seznamTekmovalcev.isEmpty()) {
            System.out.println("Seznam tekmovalcev je prazen.");
            return;
        }

        System.out.println("Seznam tekmovalcev in njihovih tock:");
        for (Tekmovalec tekmovalec : seznamTekmovalcev) {
            System.out.printf("%s: %dt\n", tekmovalec, steviloTock(tekmovalec.getDrzava()));
        }
    }

    Tekmovalec getZmagovalec() {
        int maxTocke = 0;
        Tekmovalec maxTekmovalec = null;
        for (Tekmovalec tekmovalec : seznamTekmovalcev) {
            int tocke = steviloTock(tekmovalec.getDrzava());
            if (tocke > maxTocke) {
                maxTekmovalec = tekmovalec;
                maxTocke = tocke;
            }
        }
        return maxTekmovalec;
    }

    void urediPoTockah() {
        seznamTekmovalcev.sort((Tekmovalec a, Tekmovalec b) ->
                steviloTock(b.getDrzava()) - steviloTock(a.getDrzava())
        );
        urejeno = true;
    }

    int getMesto(String drzava) {
        if (!urejeno) {
            urediPoTockah();
        }

        for (Tekmovalec tekmovalec : seznamTekmovalcev) {
            if (tekmovalec.getDrzava().equals(drzava)) {
                return seznamTekmovalcev.indexOf(tekmovalec) + 1;
            }
        }

        return -1;
    }

    void izpisiRezultateUrejeno(int n) {
        if (!urejeno) {
            urediPoTockah();
        }

        System.out.println("Najboljse uvrsceni tekmovalci:");
        int i = 1;
        for (Tekmovalec tekmovalec : seznamTekmovalcev) {
            if (i > n) break;
            System.out.printf("%d. %s: %dt\n", i, tekmovalec, steviloTock(tekmovalec.getDrzava()));
            i++;
        }
    }
}

interface Kriterij {
    int steviloTock(Tekmovanje tekmovanje, String drzava);
}

class OsnovniKriterij implements Kriterij {
    @Override
    public int steviloTock(Tekmovanje tekmovanje, String drzava) {
        int tocke = 0;
        for (Glas glas : tekmovanje.getSeznamGlasov()) {
            if (glas.getZaDrzavo().equals(drzava)) {
                tocke += glas.getStTock();
            }
        }
        return tocke;
    }
}

class LocenGlas extends Glas {
    private int stTockGlasovi, stTockZirija;

    public LocenGlas(String odDrzave, String zaDrzavo, int stTock, int stTockGlasovi, int stTockZirija) {
        super(odDrzave, zaDrzavo, stTock);
        this.stTockGlasovi = stTockGlasovi;
        this.stTockZirija = stTockZirija;
    }

    public int getStTockGlasovi() {
        return stTockGlasovi;
    }

    public int getStTockZirija() {
        return stTockZirija;
    }
}

class UtezeniKriterij implements Kriterij {
    private float utezGlasovanja, utezZirija;

    UtezeniKriterij(float utezGlasovanja, float utezZirije) {
        this.utezGlasovanja = utezGlasovanja;
        this.utezZirija = utezZirije;
    }

    @Override
    public int steviloTock(Tekmovanje tekmovanje, String drzava) {
        float tocke = 0;
        for (Glas glas : tekmovanje.getSeznamGlasov()) {
            if (glas.getZaDrzavo().equals(drzava)) {
                if (glas instanceof LocenGlas locenGlas) {
                    tocke += locenGlas.getStTockZirija() * utezZirija
                            + locenGlas.getStTockGlasovi() * utezGlasovanja;
                } else {
                    tocke += glas.getStTock() * utezZirija;
                }
            }
        }
        return Math.round(tocke);
    }
}

class ZgodovinaTekmovanj {
    private ArrayList<Tekmovanje> seznamTekmovanj;

    ZgodovinaTekmovanj(ArrayList<Tekmovanje> seznamTekmovanj) {
        this.seznamTekmovanj = seznamTekmovanj;
    }

//    public static ZgodovinaTekmovanj izDatotek(String datotekaTekmovalci, String datotekaGlasovi) {
//        Scanner scannerTekmovalci, scannerGlasovi;
//
//        try {
//            scannerTekmovalci = new Scanner(new File(datotekaTekmovalci));
//            scannerGlasovi = new Scanner(new File(datotekaGlasovi));
//        } catch (FileNotFoundException e) {
//            return null;
//        }
//
//        scannerTekmovalci.nextLine();
//        scannerGlasovi.nextLine();
//
//        ArrayList<Tekmovalec> seznamTekmovalcev = new ArrayList<>();
//        while (scannerTekmovalci.hasNextLine()) {
//            String[] fields = scannerTekmovalci.nextLine().split(";");
//            seznamTekmovalcev.add(new Tekmovalec(fields[1], fields[2], fields[3]));
//        }
//
//        ArrayList<Glas> seznamGlasov = new ArrayList<>();
//        while (scannerGlasovi.hasNextLine()) {
//            String[] fields = scannerGlasovi.nextLine().split(";");
//            switch (fields.length) {
//                case 5:
//                    seznamGlasov.add(new Glas(fields[2], fields[3], Integer.parseInt(fields[4])));
//                    break;
//                case 6:
//                    seznamGlasov.add(new LocenGlas(fields[2], fields[3], Integer.parseInt(fields[4]),
//                            Integer.parseInt(fields[5]), 0));
//                    break;
//                case 7:
//                    seznamGlasov.add(new LocenGlas(fields[2], fields[3], Integer.parseInt(fields[4]),
//                            Integer.parseInt(fields[5]), Integer.parseInt(fields[6])));
//                    break;
//            }
//        }
//    }
    
}